@@ -35,6 +35,7 @@ try:
     from gtktree.model import RowObjectTreeStore, DefaultTreeNode
     from gtktree.util  import TreeViewUtils
 except ImportError:
+    sys.excepthook (*sys.exc_info ())
     sys.exit ("Please build Py-gtktree by executing 'python setup.py build' first")
 
 
@@ -79,15 +80,19 @@ tree_view.append_column (gtk.TreeViewColumn (None, renderer,
 renderer = gtk.CellRendererText ()
 TreeViewUtils.enable_renderer_editing (renderer, store, 'name')
 
-tree_view.append_column (gtk.TreeViewColumn ('Name', renderer,
-                                             text = store.columns ('name')))
+column = gtk.TreeViewColumn ('Name', renderer,
+                             text = store.columns ('name'))
+column.set_sort_column_id (store.columns ('name'))
+tree_view.append_column (column)
 tree_view.set_expander_column (tree_view.get_columns () [-1])
 
 renderer = gtk.CellRendererText ()
 TreeViewUtils.enable_renderer_editing (renderer, store, 'description')
 
-tree_view.append_column (gtk.TreeViewColumn ('Description', renderer,
-                                             text = store.columns ('description')))
+column = gtk.TreeViewColumn ('Description', renderer,
+                             text = store.columns ('description'))
+column.set_sort_column_id (store.columns ('description'))
+tree_view.append_column (column)
 
 
 #-- EXAMPLE: Various tree editing functions --------------------------
@@ -130,6 +135,37 @@ def on_delete_selected (*ignored):
             tree_view.set_cursor (to_select.compute_path ())
 
 
+def on_shuffle (*ignored):
+    if warn_if_model_is_sorted ():
+        return
+
+    # Note that we cannot directly shuffle node children.
+    def shuffle_node (node):
+        order = range (node.num_child_nodes)
+        random.shuffle (order)
+        node.child_nodes.reorder (order)
+
+    node = TreeViewUtils.get_selected_node (tree_view)
+    if node is not None:
+        shuffle_node (node)
+    else:
+        for node in tuple (store.root.depth_first_order ()):
+            shuffle_node (node)
+
+
+def warn_if_model_is_sorted ():
+    if store.is_sorted:
+        message = gtk.MessageDialog (parent = main_window, buttons = gtk.BUTTONS_OK)
+        message.props.text       = "<b>Cannot reorder nodes in a sorted model</b>"
+        message.props.use_markup = True
+
+        message.run ()
+        message.destroy ()
+
+    return store.is_sorted
+
+
+
 #-- General UI setup -------------------------------------------------
 
 ui.connect_signals (locals ())
