@@ -41,6 +41,14 @@ class TreeNode (object):
     @property
     def parent_node (self): raise NotImplementedError
 
+    def is_ancestor (self, node):
+        while node is not None:
+            node = node.parent_node
+            if node is self:
+                return True
+
+        return False
+
     # Default implementation does nothing: if you have other way of deriving parent node
     # for the property, you can do so without implementing this method.
     def _set_parent_node (self, parent_node):
@@ -78,7 +86,7 @@ class TreeNode (object):
         return self.has_child_nodes_currently
 
     @property
-    def num_child_nodes (self):
+    def num_child_nodes_currently (self):
         child        = self.first_child_node
         num_children = 0
 
@@ -88,6 +96,10 @@ class TreeNode (object):
 
         return num_children
 
+    @property
+    def num_child_nodes (self):
+        return self.num_child_nodes_currently
+
     @property
     def first_child_node (self): raise NotImplementedError
 
@@ -124,6 +136,16 @@ class TreeNode (object):
 
         return index
 
+    def build_child_node_list (self):
+        child_nodes = []
+
+        node = self.first_child_node
+        while node is not None:
+            child_nodes.append (node)
+            node = node.next_node
+
+        return child_nodes
+
 
     def get_tree_model (self):
         node   = self
@@ -169,6 +191,9 @@ class TreeNode (object):
     def insert_child_node (self, child, before = None):
         raise NotImplementedError
 
+    def reorder_child_nodes (self, new_order, internal_model_request = False):
+        raise NotImplementedError
+
 
     @classmethod
     def create_tree (cls, data):
@@ -189,6 +214,24 @@ class TreeNode (object):
 
         return node
 
+    @classmethod
+    def create_forest (cls, *data):
+        forest = []
+        for subdata in data:
+            forest.append (cls.create_tree (subdata))
+
+        return forest
+
+    def build_row_object_skeleton (self):
+        skeleton = [self.row_object]
+
+        node = self.first_child_node
+        while node is not None:
+            skeleton.append (node.build_row_object_skeleton ())
+            node = node.next_node
+
+        return skeleton
+
 
     def hierarchic_copy (self):
         copy = type (self) ()
@@ -311,6 +354,17 @@ class TreeNode (object):
                 node = node.next_node
 
 
+    def _validate_new_child (self, child):
+        if child.parent_node is not None:
+            raise ValueError ("new child nodes cannot already have a parent node")
+        if child is self:
+            raise ValueError ("cannot make a node its own child")
+        if child.is_ancestor (self):
+            raise ValueError ("cannot make a parent or non-direct parent a child node")
+        if child._do_get_tree_model () is not None:
+            raise ValueError ("root node of a model's tree cannot become a child node")
+
+
     def __repr__(self):
         return '<%s.%s at 0x%x: %r>' % (type (self).__module__, type (self).__name__, id (self), self.row_object)
 
@@ -337,6 +391,11 @@ class DefaultTreeNode (TreeNode):
     def _set_row_object (self, row_object):
         self._row_object = row_object
 
+        model = self.get_tree_model ()
+        if model is not None:
+            model.note_changes (self)
+
+
     row_object = property (_get_row_object, _set_row_object)
     del _get_row_object, _set_row_object
 
@@ -365,6 +424,13 @@ class DefaultTreeNode (TreeNode):
     del _get_child_nodes, _set_child_nodes
 
 
+    # FIXME: Make the returned list a view over 'child_nodes' that reflects any
+    #        changes. *Don't* document this property until that is done.
+    @property
+    def child_row_objects (self):
+        return [child.row_object for child in self.child_nodes]
+
+
     @property
     def next_node (self):
         return self.__next_node
@@ -378,7 +444,7 @@ class DefaultTreeNode (TreeNode):
         return bool (self.__child_nodes)
 
     @property
-    def num_child_nodes (self):
+    def num_child_nodes_currently (self):
         child_nodes = self.__child_nodes
         return len (child_nodes) if child_nodes is not None else 0
 
@@ -403,6 +469,9 @@ class DefaultTreeNode (TreeNode):
 
         raise ValueError
 
+    def build_child_node_list (self):
+        return self.child_nodes[:]
+
 
     def delete_child_node (self, child):
         self.child_nodes.remove (child)
@@ -413,6 +482,9 @@ class DefaultTreeNode (TreeNode):
         else:
             self.child_nodes.insert (self.child_nodes.index (before), child)
 
+    def reorder_child_nodes (self, new_order, internal_model_request = False):
+        self.child_nodes.reorder (new_order, internal_model_request)
+
 
 
 class _DefaultTreeNodeChildren (list):
@@ -518,27 +590,39 @@ class _DefaultTreeNodeChildren (list):
         return num_removed
 
     def _do_insert (self, model, path, index, nodes):
-        nodes = tuple (nodes)
+        self_node = self.__node
+        nodes     = tuple (nodes)
 
         for child in nodes:
             if not isinstance (child, TreeNode):
                 raise TypeError ("can only use TreeNode objects as child nodes")
-            if child.parent_node is not None:
-                raise ValueError ("new child nodes cannot already have a parent node")
-            if child._do_get_tree_model () is not None:
-                raise ValueError ("root node of a model's tree cannot become a child node")
+            self_node._validate_new_child (child)
 
-        current_child = (self[index] if index < len (self) else None)
+        insert_sorted = (False if model is None else model.is_sorted)
+        if not insert_sorted:
+            current_child = (self[index] if index < len (self) else None)
 
         for child in nodes:
+            if insert_sorted:
+                # Original 'index' argument is just ignored.
+                if self:
+                    index         = model._compute_child_node_insertion_point (self, child)
+                    current_child = (self[index] if index < len (self) else None)
+                else:
+                    index         = 0
+                    current_child = None
+
+                if child.has_child_nodes:
+                    model._do_apply_sort_settings_to_node (child)
+
             list.insert (self, index, child)
 
             if index > 0:
                 self[index - 1]._set_next_node (child)
             else:
-                self.__node._set_first_child_node (child)
+                self_node._set_first_child_node (child)
 
-            child._set_parent_node (self.__node)
+            child._set_parent_node (self_node)
             child._set_next_node (current_child)
 
             if model is not None:
@@ -547,21 +631,51 @@ class _DefaultTreeNodeChildren (list):
                 model.row_inserted (child_path, child_iter)
 
                 if len (self) == 1 and path:
-                    model.row_has_child_toggled (path, model._do_get_iter (self.__node))
+                    model.row_has_child_toggled (path, model._do_get_iter (self_node))
 
                 if child.has_child_nodes_currently:
                     model.row_has_child_toggled (child_path, child_iter)
 
             index += 1
 
-    def _get_tree_model_and_node_path (self):
-        model = self.__node.get_tree_model ()
-        if model is None:
-            return None, None
-        elif self.__node is not model.root:
-            return model, model.get_path (model._do_get_iter (self.__node))
-        else:
-            return model, ()
+
+    def reorder (self, new_order, internal_model_request = False):
+        model, path = self._get_tree_model_and_node_path ()
+        if not internal_model_request and model is not None:
+            model._check_is_not_sorted ()
+
+        new_order = list (new_order)
+        if len (new_order) != len (self):
+            raise ValueError ("'new_order' size (%d) is not equal to number of children (%d)"
+                              % (len (new_order), len (self)))
+
+        if not self:
+            return
+
+        is_copied = [False] * len (self)
+        reordered = [None]  * len (self)
+
+        for to_, from_ in enumerate (new_order):
+            from_          = self._get_existing_child_index (from_)
+            new_order[to_] = from_
+
+            if not is_copied[from_]:
+                reordered[to_]   = self[from_]
+                is_copied[from_] = True
+            else:
+                raise ValueError ("child node at %d has to be moved into several positions at once"
+                                  % from_)
+
+        list.__setitem__(self, slice (len (self)), reordered)
+
+        self.__node._set_first_child_node (self[0])
+        for k in range (len (self) - 1):
+            self[k]._set_next_node (self[k + 1])
+
+        self[-1]._set_next_node (None)
+
+        if model is not None:
+            model.rows_reordered (path, model.get_iter (path) if path else None, new_order)
 
 
     def __delitem__(self, index):
@@ -601,6 +715,16 @@ class _DefaultTreeNodeChildren (list):
                                   % (len (new_children), len (indices)))
 
 
+    def _get_tree_model_and_node_path (self):
+        model = self.__node.get_tree_model ()
+        if model is None:
+            return None, None
+        elif self.__node is not model.root:
+            return model, model.get_path (model._do_get_iter (self.__node))
+        else:
+            return model, ()
+
+
     # These methods are deprecated, but we need to override them as long as underlying
     # Python 'list' class has them.
 
