@@ -23,6 +23,8 @@
 import gtk
 import operator
 
+from bisect import bisect_right
+
 
 __all__ = ('RowObjectTreeModel',
            'RowObjectListStore',
@@ -81,11 +83,23 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
     def columns (self):
         return self.__columns
 
+    def _get_column_getter (self, index):
+        return self.__getters[index]
+
+
     @property
     def node_based (self):
         return False
 
 
+    @property
+    def root (self):
+        if self.node_based:
+            raise NotImplementedError
+        else:
+            raise TypeError ("only node-based models can have a root node")
+
+
     # Strictly speaking, this property only makes sense for node-based model.
 
     def _get_default_node_type (self):
@@ -139,6 +153,15 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
         raise NotImplementedError
 
 
+    @property
+    def is_sorted (self):
+        return False
+
+    def _check_is_not_sorted (self):
+        if self.is_sorted:
+            raise RuntimeError ("the model must not be user-sorted")
+
+
     def note_changes (self, *rows):
         if not rows:
             return 0
@@ -165,6 +188,19 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
         return num_changed
 
 
+    def _compute_child_node_insertion_point (self, child_nodes, new_child):
+        if self.node_based:
+            raise NotImplementedError
+        else:
+            raise TypeError ("can only be called on node-based models")
+
+    def _do_apply_sort_settings_to_node (self, node):
+        if self.node_based:
+            raise NotImplementedError
+        else:
+            raise TypeError ("can only be called on node-based models")
+
+
     def _get_object_type (self):
         return self.__object_type
 
@@ -231,6 +267,103 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
 
 
 
+if not hasattr (gtk, 'TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID'):
+    from gtktree._impl import c_hacks
+
+    _USING_C_HACKS = True
+    _UNSORTED      = c_hacks.TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
+    _DEFAULT_SORT  = c_hacks.TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID
+
+else:
+    _USING_C_HACKS = False
+    _UNSORTED      = gtk.TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
+    _DEFAULT_SORT  = gtk.TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID
+
+
+
+# A (currently internal) mixin that helps with implementing gtk.TreeSortable interface.
+# Note that it _must_ be listed before anything inheriting gtk.TreeModel for correct MRO.
+# Relies on inherited class to have 'columns' property and _do_apply_sort_settings()
+# method.
+class _TreeSortableHelper (object):
+
+    def __init__(self):
+        self._sort_column_id = _UNSORTED
+        self._sort_ascending = True
+
+        if self._support_classic_sort_functions:
+            self.__default_sort_function = None
+            self.__sort_functions        = []
+
+
+    @property
+    def is_sorted (self):
+        return self._sort_column_id != _UNSORTED
+
+    @property
+    def _support_classic_sort_functions (self):
+        return True
+
+
+    def _get_current_sort_function (self):
+        if self._sort_column_id != _UNSORTED and self._support_classic_sort_functions:
+            if self._sort_column_id == _DEFAULT_SORT:
+                return self.__default_sort_function
+            elif self._sort_column_id < len (self.__sort_functions):
+                return self.__sort_functions[self._sort_column_id]
+
+        return None        
+
+
+    def do_get_sort_column_id (self):
+        return (self._sort_column_id,
+                gtk.SORT_ASCENDING if self._sort_ascending else gtk.SORT_DESCENDING)
+
+
+    def do_set_sort_column_id (self, sort_column_id, sort_ascending):
+        sort_ascending = (sort_ascending == gtk.SORT_ASCENDING)
+        if self._sort_column_id == sort_column_id and self._sort_ascending == sort_ascending:
+            return
+
+        self._sort_column_id = sort_column_id
+        self._sort_ascending = sort_ascending
+
+        self.sort_column_changed ()
+
+        if sort_column_id != _UNSORTED:
+            self._do_apply_sort_settings ()
+
+
+    def do_set_sort_func (self, sort_column_id, func):
+        if not self._support_classic_sort_functions:
+            raise RuntimeError ("sort functions are not supported in this model")
+
+        if 0 <= sort_column_id and sort_column_id < len (self.columns):
+            if len (self.__sort_functions) <= sort_column_id:
+                self.__sort_functions.extend ([None] * (1 + sort_column_id
+                                                        - len (self.__sort_functions)))
+
+            self.__sort_functions[sort_column_id] = func
+            if self._sort_column_id == sort_column_id:
+                self._do_apply_sort_settings ()
+
+        else:
+            raise IndexError
+
+
+    def do_has_default_sort_func (self):
+        return self._support_classic_sort_functions and self.__default_sort_function is not None
+
+    def do_set_default_sort_func (self, func):
+        if not self._support_classic_sort_functions:
+            raise RuntimeError ("sort functions are not supported in this model")
+
+        self.__default_sort_function = func
+        if self._sort_column_id == _DEFAULT_SORT:
+            self._do_apply_sort_settings ()
+
+
+
 # IMPLEMENTATION NOTES
 #
 # Iterator user data are indices of the rows (i.e., 0, ..., N-1 where N is list length).
@@ -238,11 +371,9 @@ class RowObjectTreeModel (gtk.GenericTreeModel):
 # iterators, otherwise there can be refcount corruption.  Because we have own persistent
 # references to iterator user data, we can set 'leak_references' to False.  Note that
 # iterators don't survive list store modifications except for appending to the end.
-#
-# It would be nice to implement gtk.TreeSortable, however PyGTK doesn't allow us to do so
-# yet.
 
-class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDest):
+class RowObjectListStore (_TreeSortableHelper, RowObjectTreeModel,
+                          gtk.TreeSortable, gtk.TreeDragSource, gtk.TreeDragDest):
 
     __gtype_name__ = 'RowObjectListStore'
 
@@ -255,6 +386,7 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
             self._indices = range (len (self._values))
 
         RowObjectTreeModel.__init__(self, columns)
+        _TreeSortableHelper.__init__(self)
 
         self.props.leak_references = False
 
@@ -294,10 +426,14 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
     def extend (self, row_objects):
         self._do_insert (len (self._values), row_objects)
 
+
     def _do_insert (self, index, row_objects, after = False):
         if not row_objects:
             return
 
+        if self.is_sorted:
+            return self._do_insert_sorted (row_objects)
+
         values  = self._values
         indices = self._indices
 
@@ -328,10 +464,82 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
                 index += 1
 
 
+    def _do_insert_sorted (self, row_objects):
+        compare = self._get_current_sort_function ()
+
+        values         = self._values
+        indices        = self._indices
+        sort_ascending = self._sort_ascending
+
+        self.invalidate_iters ()
+        indices.extend (range (len (indices), len (indices) + len (row_objects)))
+
+        # FIXME: Really need to think it out.  Currently, this is quite ugly.
+        if compare is not None:
+            for row_object in row_objects:
+                values.append (row_object)
+                index = self.__bisect_with_classic_sort_function (compare)
+
+                if index < len (values) - 1:
+                    del values[-1]
+                    values.insert (index, row_object)
+
+                last_iter = self.get_iter (index)
+                self.row_inserted (index, last_iter)
+
+        else:
+            # FIXME: Suboptimal for larger lists: we don't really need to get all values.
+            #        Make this lazy.
+            getter      = self._get_column_getter (self._sort_column_id)
+            sort_values = [getter (value) for value in values]
+
+            if not sort_ascending:
+                sort_values.reverse ()
+
+            for row_object in row_objects:
+                value = getter (row_object)
+                index = bisect_right (sort_values, value)
+
+                sort_values.insert (index, value)
+
+                if not sort_ascending:
+                    index = len (values) - index
+
+                values.insert (index, row_object)
+
+                last_iter = self.get_iter (index)
+                self.row_inserted (index, last_iter)
+
+        if len (row_objects) == 1:
+            return last_iter
+
+
+    # Currently assumes that the new row object is at the last position.
+    def __bisect_with_classic_sort_function (self, compare):
+        if len (self) == 1:
+            return 0
+
+        from_    = 0
+        to_      = len (self) - 1
+        new_iter = self.create_tree_iter (to_)
+
+        while from_ < to_:
+            compare_to = (from_ + to_) / 2
+            result     = compare (self, new_iter, self.create_tree_iter (compare_to))
+
+            if result > 0:
+                from_ = compare_to + 1
+            elif result < 0:
+                to_ = compare_to
+            else:
+                return compare_to
+
+        return from_
+
+
     def remove (self, row_object):
         del self[self._values.index (row_object)]
 
-
     def remove_at (self, *indices):
         if not indices:
             return
@@ -381,6 +589,8 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
 
 
     def reorder (self, new_order):
+        self._check_is_not_sorted ()
+
         # Since we will (possibly) modify the list below, we make a copy.  Also, this
         # allows to accept any iterable argument.
         new_order = list (new_order)
@@ -408,6 +618,7 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
 
 
     def reverse (self):
+        self._check_is_not_sorted ()
         if not self._values:
             return
 
@@ -421,32 +632,21 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
 
 
     def sort (self, cmp = None, key = None, reverse = False):
-        if len (self._values) <= 1:
+        self._check_is_not_sorted ()
+        if len (self) <= 1:
             return
 
-        # FIXME: No longer need to emulate 'key' as we require Python 2.5.  Simplify.
-        if key is None:
-            values = [(value, index, value) for index, value in enumerate (self._values)]
-        else:
-            values = [(key (value), index, value) for index, value in enumerate (self._values)]
+        values = zip (self._values, range (len (self)))
+        values.sort (cmp     = (None if cmp is None else (lambda a, b: cmp (a[0], b[0]))),
+                     key     = (None if key is None else (lambda a: key (a[0]))),
+                     reverse = reverse)
 
-        if cmp is None:
-            values.sort (reverse = reverse)
-        else:
-            values.sort (cmp = lambda a, b: cmp (a[0], b[0]), reverse = reverse)
-
-        self.invalidate_iters ()
-        self._values = [value for key, index, value in values]
-
-        new_order = [False] * len (values)
-        for new_index, data in enumerate (values):
-            key, index, value  = data
-            new_order[index] = new_index
-
-        self.rows_reordered ((), None, new_order)
+        self._handle_new_value_order (values)
 
 
     def swap (self, index1, index2):
+        self._check_is_not_sorted ()
+
         index1 = self._get_existing_row_index (index1)
         index2 = self._get_existing_row_index (index2)
 
@@ -465,10 +665,13 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
 
 
     def move_to (self, from_index, to_index):
+        self._check_is_not_sorted ()
         self._do_move (self._get_existing_row_index (from_index),
                        min (self._get_insertion_point (to_index), len (self._values) - 1))
 
     def move_before (self, from_index, before_index):
+        self._check_is_not_sorted ()
+
         from_index   = self._get_existing_row_index (from_index)
         before_index = self._get_existing_row_index (before_index)
 
@@ -481,6 +684,8 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
                               % from_index)
 
     def move_after (self, from_index, after_index):
+        self._check_is_not_sorted ()
+
         from_index  = self._get_existing_row_index (from_index)
         after_index = self._get_existing_row_index (after_index)
 
@@ -500,6 +705,28 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
 
             self.rows_reordered ((), None, new_order)
 
+    def _handle_new_value_order (self, reordered_values):
+        self.invalidate_iters ()
+        self._values = [value for value, index in reordered_values]
+
+        new_order = [False] * len (self)
+        for new_index, entry in enumerate (reordered_values):
+            value, index         = entry
+            new_order[new_index] = index
+
+        self.rows_reordered ((), None, new_order)
+
+
+    def note_changes (self, *rows):
+        num_changed = super (RowObjectListStore, self).note_changes (*rows)
+
+        # FIXME: Really non-optimal.  We don't need to resort all the model if a few
+        #        (especially one) row has been changed.
+        if self.is_sorted and num_changed > 0:
+            self._do_apply_sort_settings ()
+
+        return num_changed
+
 
     def __len__(self):
         return len (self._values)
@@ -523,6 +750,11 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
             index = self._get_existing_row_index (index)
             self._values[index] = row_object
             self.row_changed (index, self.get_iter (index))
+
+            # FIXME: Really non-optimal.  We don't need to resort all the model.
+            if self.is_sorted:
+                self._do_apply_sort_settings ()
+
             return
 
         row_objects       = tuple (row_object)
@@ -535,6 +767,11 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
                 self._values[_index] = row_objects[k]
                 self.row_changed (_index, self.get_iter (_index))
 
+            # FIXME: Really non-optimal.  We don't need to resort all the model if a few
+            #        (especially one) row has been changed.
+            if self.is_sorted:
+                self._do_apply_sort_settings ()
+
             if len (row_objects) > common_length:
                 self._do_insert (start + common_length, row_objects[common_length:])
             elif stop - start > common_length:
@@ -545,6 +782,11 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
                 for index, row_object in zip (indices, row_objects):
                     self._values[index] = row_object
                     self.row_changed (index, self.get_iter (index))
+
+                # FIXME: Really non-optimal.  We don't need to resort all the model if a
+                #        few (especially one) row has been changed.
+                if self.is_sorted:
+                    self._do_apply_sort_settings ()
             else:
                 raise ValueError (('attempt to assign sequence of size %d '
                                    'to extended slice of size %d')
@@ -655,6 +897,28 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
             return None
 
 
+    # Implementing gtk.TreeSortable.  Note that this relies on support from
+    # _TreeSortableHelper and from our internal 'c_hacks' module if needed (on older
+    # PyGTK).
+
+    def _do_apply_sort_settings (self):
+        values = zip (self._values, range (len (self)))
+
+        compare = self._get_current_sort_function ()
+
+        if compare is not None:
+            values.sort (cmp     = lambda a, b: compare (self,
+                                                         self.create_tree_iter (a[1]),
+                                                         self.create_tree_iter (b[1])),
+                         reverse = not self._sort_ascending)
+        elif self._sort_column_id != _DEFAULT_SORT:
+            getter = self._get_column_getter (self._sort_column_id)
+            values.sort (key     = lambda a: getter (a[0]),
+                         reverse = not self._sort_ascending)
+
+        self._handle_new_value_order (values)
+
+
     # Implementing gtk.TreeDragSource.
 
     def do_row_draggable (self, path):
@@ -680,9 +944,18 @@ class RowObjectListStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
             return False
 
     def do_row_drop_possible (self, dest_path, selection_data):
+        if self.is_sorted:
+            source_model, source_path, row_object = self._get_drag_data (selection_data)
+            if source_model is self:
+                return False
+
         return True
 
 
+if _USING_C_HACKS:
+    c_hacks.register_tree_sortable_implementation (RowObjectListStore)
+
+
 
 from gtktree._impl.tree_node import TreeNode, DefaultTreeNode, LazyDefaultTreeNode
 
@@ -697,7 +970,8 @@ LazyDefaultTreeNode.__module__ = __name__
 # 'leak_references' to False.  Also, by definition, iterators survive all tree store
 # modifications.
 
-class RowObjectTreeStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDest):
+class RowObjectTreeStore (_TreeSortableHelper, RowObjectTreeModel,
+                          gtk.TreeSortable, gtk.TreeDragSource, gtk.TreeDragDest):
 
     __gtype_name__ = 'RowObjectTreeStore'
 
@@ -707,6 +981,7 @@ class RowObjectTreeStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
         self.__root.row_object = self
 
         RowObjectTreeModel.__init__(self, columns)
+        _TreeSortableHelper.__init__(self)
 
         self.props.leak_references = False
 
@@ -721,13 +996,28 @@ class RowObjectTreeStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
         return self.__root
 
 
+    def note_changes (self, *rows):
+        num_changed = super (RowObjectTreeStore, self).note_changes (*rows)
+
+        if self.is_sorted:
+            unique_parents = set ()
+            for row in rows:
+                unique_parents.add (self.get_node (self._do_get_iter (row)).parent_node)
+
+            self._do_apply_sort_settings (unique_parents)
+
+        return num_changed
+
+
     def __len__(self):
         # Default value comes from gtk.GenericTreeModel.
         raise TypeError ("object of type '%s' has no len()" % type (self).__name__)
 
-
     def __contains__(self, row_object):
-        # FIXME
+        for node in self.root.depth_first_order (False):
+            if node.row_object == row_object:
+                return True
+
         return False
 
     def __iter__(self):
@@ -785,6 +1075,58 @@ class RowObjectTreeStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
             return self.root.get_child_node (n)
 
 
+    def _compute_child_node_insertion_point (self, child_nodes, new_child):
+        getter         = self._get_column_getter (self._sort_column_id)
+        sort_values    = [getter (node.row_object) for node in child_nodes]
+        sort_ascending = self._sort_ascending
+
+        if not sort_ascending:
+            sort_values.reverse ()
+
+        index = bisect_right (sort_values, getter (new_child.row_object))
+        return index if sort_ascending else len (child_nodes) - index
+
+
+    # Implementing gtk.TreeSortable.  Note that this relies on support from
+    # _TreeSortableHelper and from our internal 'c_hacks' module if needed (on older
+    # PyGTK).
+
+    # FIXME: Should support eventually.
+    @property
+    def _support_classic_sort_functions (self):
+        return False
+
+    def _do_apply_sort_settings_to_node (self, node):
+        self._do_apply_sort_settings (tuple (child for child in node.depth_first_order ()
+                                             if child.has_child_nodes_currently))
+
+    def _do_apply_sort_settings (self, nodes = None):
+        if self._sort_column_id != _DEFAULT_SORT:
+            getter          = self._get_column_getter (self._sort_column_id)
+            sort_descending = not self._sort_ascending
+
+            if nodes is None:
+                # Note: we are making a copy of the order since there will be structural
+                # modifications.
+                nodes = tuple (self.root.depth_first_order ())
+
+            for node in nodes:
+                num_child_nodes = node.num_child_nodes_currently
+                if num_child_nodes < 2:
+                    continue
+
+                child_nodes = zip (node.build_child_node_list (), range (num_child_nodes))
+                child_nodes.sort (key     = lambda a: getter (a[0].row_object),
+                                  reverse = sort_descending)
+
+                new_order = [False] * num_child_nodes
+                for new_index, entry in enumerate (child_nodes):
+                    value, index         = entry
+                    new_order[new_index] = index
+
+                node.reorder_child_nodes (new_order, True)
+
+
     # Implementing gtk.TreeDragSource.
 
     def do_row_draggable (self, path):
@@ -817,6 +1159,12 @@ class RowObjectTreeStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
             return False
 
         dest_parent, dest_before = self._get_drag_destination_nodes (dest_path)
+        if node is dest_parent or node.is_ancestor (dest_parent):
+            return False
+
+        if self.is_sorted and node.parent_node == dest_parent:
+            return False
+
         return (dest_parent.accepts_dragged_child_node (node)
                 and node.accepts_parent_node_after_drag (dest_parent))
 
@@ -836,6 +1184,10 @@ class RowObjectTreeStore (RowObjectTreeModel, gtk.TreeDragSource, gtk.TreeDragDe
             return self.root, self.root.first_child_node
 
 
+if _USING_C_HACKS:
+    c_hacks.register_tree_sortable_implementation (RowObjectTreeStore)
+
+
 
 class _RootTreeNode (DefaultTreeNode):
 
