@@ -27,6 +27,8 @@ if __name__ == '__main__':
 
 import unittest
 
+import gtk
+
 from test._common  import Foo, PopAllList
 from gtktree.model import RowObjectListStore
 
@@ -338,27 +340,27 @@ class RowObjectListStoreTestCase (unittest.TestCase):
         store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort ()
         self.assertEqual (store[:], ['A', 'C', 'D', 'b', 'e'])
-        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [1, 4, 3, 0, 2])])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [3, 0, 4, 2, 1])])
 
         store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (key = str.lower)
         self.assertEqual (store[:], ['A', 'b', 'C', 'D', 'e'])
-        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [2, 4, 1, 0, 3])])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [3, 2, 0, 4, 1])])
 
         store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (cmp = lambda a, b: -cmp (a, b))
         self.assertEqual (store[:], ['e', 'b', 'D', 'C', 'A'])
-        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [2, 0, 3, 4, 1])])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [1, 2, 4, 0, 3])])
 
         store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (cmp = lambda a, b: -cmp (a, b), key = str.lower)
         self.assertEqual (store[:], ['e', 'D', 'C', 'b', 'A'])
-        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [3, 0, 1, 4, 2])])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [1, 4, 0, 2, 3])])
 
         store, emissions = _create_foo_list_store (['C', 'e', 'b', 'A', 'D'])
         store.sort (reverse = True)
         self.assertEqual (store[:], ['e', 'b', 'D', 'C', 'A'])
-        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [2, 0, 3, 4, 1])])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [1, 2, 4, 0, 3])])
 
 
     def test_swap (self):
@@ -440,6 +442,91 @@ class RowObjectListStoreTestCase (unittest.TestCase):
             self.assertRaises (expected_result, lambda: mover (store))
 
 
+    def test_sortable_interface_1 (self):
+        store, emissions = _create_sorting_foo_list_store (already_sorted = False)
+
+        self.assert_(not store.is_sorted)
+
+        store.set_sort_column_id (0, gtk.SORT_ASCENDING)
+        self.assert_     (store.is_sorted)
+        self.assertEqual (store[:], [Foo (n) for n in range (6)])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [1, 5, 4, 0, 2, 3])])
+
+        store.set_sort_column_id (0, gtk.SORT_DESCENDING)
+        self.assert_     (store.is_sorted)
+        self.assertEqual (store[:], [Foo (n) for n in range (5, -1, -1)])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [5, 4, 3, 2, 1, 0])])
+
+
+    # Test that it 'appends' at the correct position.
+    def test_sortable_interface_append (self):
+        store, emissions = _create_sorting_foo_list_store ()
+
+        store.append (Foo (1.5))
+        self.assertEqual (store[2], Foo (1.5))
+        self.assertEqual (emissions.pop_all (), [('inserted', (2,), 2)])
+
+
+    def test_sortable_interface_set_item (self):
+        store, emissions = _create_sorting_foo_list_store ()
+
+        store[4] = Foo (-1)
+
+        self.assertEqual (store[:], [Foo (-1), Foo (0), Foo (1), Foo (2), Foo (3), Foo (5)])
+        self.assertEqual (emissions.pop_all (), [('changed', (4,), 4),
+                                                 ('reordered', '?' or [4, 0, 1, 2, 3, 5])])
+
+
+    def test_sortable_interface_note_changes (self):
+        store, emissions = _create_sorting_foo_list_store ()
+
+        store[2].foo = 4.5
+        store.note_changes (2)
+
+        self.assertEqual (store[:], [Foo (0), Foo (1), Foo (3), Foo (4), Foo (4.5), Foo (5)])
+        self.assertEqual (emissions.pop_all (), [('changed', (2,), 2),
+                                                 ('reordered', '?' or [0, 1, 3, 4, 2, 5])])
+
+
+    def test_sortable_interface_sort_function_1 (self):
+        store, emissions = _create_sorting_foo_list_store (already_sorted = False)
+
+        self.assert_(not store.is_sorted)
+
+        store.set_sort_func (0, _sort_even_numbers_first)
+        store.set_sort_column_id (0, gtk.SORT_ASCENDING)
+        self.assert_     (store.is_sorted)
+        self.assertEqual (store[:], [Foo (0), Foo (2), Foo (4), Foo (1), Foo (3), Foo (5)])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [1, 4, 2, 5, 0, 3])])
+
+
+    def test_sortable_interface_sort_function_2 (self):
+        store, emissions = _create_sorting_foo_list_store ()
+
+        store.set_sort_func (0, _sort_even_numbers_first)
+        self.assertEqual (store[:], [Foo (0), Foo (2), Foo (4), Foo (1), Foo (3), Foo (5)])
+        self.assertEqual (emissions.pop_all (), [('reordered', '?' or [0, 2, 4, 1, 3, 5])])
+
+
+    def test_sortable_interface_sort_function_append (self):
+        store, emissions = _create_sorting_foo_list_store ()
+
+        store.set_sort_func (0, _sort_even_numbers_first)
+        emissions.pop_all ()
+
+        store.append (Foo (10))
+        self.assertEqual (store[3], Foo (10))
+        self.assertEqual (emissions.pop_all (), [('inserted', (3,), 3)])
+
+        store.append (Foo (7))
+        self.assertEqual (store[7], Foo (7))
+        self.assertEqual (emissions.pop_all (), [('inserted', (7,), 7)])
+
+        store.append (Foo (-2))
+        self.assertEqual (store[0], Foo (-2))
+        self.assertEqual (emissions.pop_all (), [('inserted', (0,), 0)])
+
+
 
 def _create_foo_list_store (values = None):
     if values is None:
@@ -472,6 +559,29 @@ def _create_foo_list_store (values = None):
     return store, emissions
 
 
+def _create_sorting_foo_list_store (values = None, already_sorted = True):
+    if values is None:
+        values = [Foo (3), Foo (0), Foo (4), Foo (5), Foo (2), Foo (1)]
+
+    store, emissions = _create_foo_list_store (values)
+
+    if already_sorted:
+        store.set_sort_column_id (0, gtk.SORT_ASCENDING)
+        emissions.pop_all ()
+
+    return store, emissions
+
+
+def _sort_even_numbers_first (model, iter1, iter2):
+    a = model[iter1].foo
+    b = model[iter2].foo
+
+    if a % 2 == b % 2:
+        return a - b
+    else:
+        return a % 2 - b % 2
+
+
 
 if __name__ == '__main__':
     unittest.main ()
