@@ -27,15 +27,17 @@ if __name__ == '__main__':
 
 import unittest
 
+import gtk
+
 from test._common  import Foo, PopAllList
 from gtktree.model import RowObjectTreeStore, TreeNode, DefaultTreeNode
 
 
 
-# Some tests assume current order of row changing, insertion or removal.  Such tests will
-# fail if this order is changed for whatever reason and those failures won't necessary
-# indicate real bugs.  In that case, you will need to check and possibly alter these
-# tests.
+# Some tests assume current order of row changing, insertion, removal or reordering (when
+# sorting).  Such tests will fail if this order is changed for whatever reason and those
+# failures won't necessary indicate real bugs.  In that case, you will need to check and
+# possibly alter these tests.
 
 class RowObjectTreeStoreTestCase (unittest.TestCase):
 
@@ -50,6 +52,66 @@ class RowObjectTreeStoreTestCase (unittest.TestCase):
         self.assertEqual (store.root.num_child_nodes, 2)
 
 
+    def test_sortable_interface_1 (self):
+        store, emissions = _create_sorting_foo_tree_store (already_sorted = False)
+
+        self.assert_(not store.is_sorted)
+
+        store.set_sort_column_id (0, gtk.SORT_ASCENDING)
+        self.assert_     (store.is_sorted)
+        self.assertEqual (store.root.build_row_object_skeleton () [1:],
+                          _sorting_foo_forest_sorted_ascending ())
+        self.assertEqual (emissions.pop_all (),
+                          [('reordered',         '?' or [0, 1, 2]),
+                           ('reordered', (0,),   '?' or [3, 4, 1, 2, 0]),
+                           ('reordered', (0, 3), '?' or [0, 1]),
+                           ('reordered', (1,),   '?' or [1, 0])])
+
+        store.set_sort_column_id (0, gtk.SORT_DESCENDING)
+        self.assert_     (store.is_sorted)
+        self.assertEqual (store.root.build_row_object_skeleton () [1:],
+                          _sorting_foo_forest_sorted_descending ())
+        self.assertEqual (emissions.pop_all (),
+                          [('reordered',         '?' or [2, 1, 0]),
+                           ('reordered', (2,),   '?' or [4, 3, 2, 1, 0]),
+                           ('reordered', (2, 1), '?' or [1, 0]),
+                           ('reordered', (1,),   '?' or [1, 0])])
+
+
+    # Test that it 'appends' at the correct position.
+    def test_sortable_interface_append (self):
+        store, emissions = _create_sorting_foo_tree_store ()
+        node             = store.root
+
+        node.child_nodes.append (DefaultTreeNode (Foo (2.5)))
+        self.assertEqual (node.child_nodes[2].row_object, Foo (2.5))
+        self.assertEqual (emissions.pop_all (), [('inserted', (2,))])
+
+
+    def test_sortable_interface_set_row_object (self):
+        store, emissions = _create_sorting_foo_tree_store ()
+        node             = store.root.child_nodes[0].child_nodes[2]
+
+        node.row_object = Foo (10.5)
+
+        self.assertEqual (node.parent_node.child_row_objects,
+                          [Foo (10), Foo (10.5), Foo (11), Foo (13), Foo (14)])
+        self.assertEqual (emissions.pop_all (), [('changed', (0, 2)),
+                                                 ('reordered', (0,), '?' or [0, 2, 1, 3, 4])])
+
+
+    def test_sortable_interface_note_changes (self):
+        store, emissions = _create_sorting_foo_tree_store ()
+        node             = store.root.child_nodes[2]
+
+        node.row_object.foo = 0.5
+        store.note_changes (node)
+
+        self.assertEqual (node.parent_node.child_row_objects, [Foo (0.5), Foo (1), Foo (2)])
+        self.assertEqual (emissions.pop_all (), [('changed', (2,)),
+                                                 ('reordered', '?' or [2, 0, 1])])
+
+
 
 class TreeNodeTestCase (unittest.TestCase):
 
@@ -328,6 +390,14 @@ class DefaultTreeNodeTestCase (unittest.TestCase):
     def test_illegal_child_3 (self):
         store, emissions = _create_foo_tree_store (())
         self.assertRaises (ValueError, lambda: DefaultTreeNode ().child_nodes.append (store.root))
+
+    def test_illegal_child_4 (self):
+        x = DefaultTreeNode (None)
+        self.assertRaises (ValueError, lambda: x.child_nodes.append (x))
+
+    def test_illegal_child_5 (self):
+        x = DefaultTreeNode.create_tree ([None, [None]])
+        self.assertRaises (ValueError, lambda: x.child_nodes[0].child_nodes.append (x))
         
 
 
@@ -363,6 +433,65 @@ def _create_foo_tree_store (nodes = None):
 
 
 
+def _sorting_foo_forest ():
+    return [[Foo (1),
+             [Foo (14)],
+             [Foo (12),
+              [Foo (120)]],
+             [Foo (13),
+              [Foo (130)],
+              [Foo (131)]],
+             [Foo (10)],
+             [Foo (11)]],
+            [Foo (3)],
+            [Foo (2),
+             [Foo (21)],
+             [Foo (20)]]]
+
+def _sorting_foo_forest_sorted_ascending ():
+    return [[Foo (1),
+             [Foo (10)],
+             [Foo (11)],
+             [Foo (12),
+              [Foo (120)]],
+             [Foo (13),
+              [Foo (130)],
+              [Foo (131)]],
+             [Foo (14)]],
+            [Foo (2),
+             [Foo (20)],
+             [Foo (21)]],
+            [Foo (3)]]
+
+def _sorting_foo_forest_sorted_descending ():
+    return [[Foo (3)],
+            [Foo (2),
+             [Foo (21)],
+             [Foo (20)]],
+            [Foo (1),
+             [Foo (14)],
+             [Foo (13),
+              [Foo (131)],
+              [Foo (130)]],
+             [Foo (12),
+              [Foo (120)]],
+             [Foo (11)],
+             [Foo (10)]]]
+
+def _create_sorting_foo_tree_store (values = None, already_sorted = True):
+    if values is None:
+        values = DefaultTreeNode.create_forest (*_sorting_foo_forest ())
+
+    store, emissions = _create_foo_tree_store (values)
+
+    if already_sorted:
+        store.set_sort_column_id (0, gtk.SORT_ASCENDING)
+        emissions.pop_all ()
+
+    return store, emissions
+
+
+
 if __name__ == '__main__':
     unittest.main ()
 
